#include <States/GameState/GameBuilder/GameBuilder.h>
#include <random>

void CommonBuilder::create_context(float dynamic_objets_ratio) {
    std::mt19937 gen(std::random_device{}());

    std::vector<std::vector<std::unique_ptr<Room>>::iterator> buf_rooms;
    for (auto& row : m_rooms) {
        for (auto room = row.begin(); room != row.end(); ++room) {
            buf_rooms.push_back(room);
        }
    }

    if (buf_rooms.empty()) return;

    std::uniform_int_distribution distrib(0 , static_cast<int>(buf_rooms.size()-1));

    auto pos = buf_rooms.begin() + distrib(gen);
    m_context.pacman.set_location((*(buf_rooms.begin() + distrib(gen)))->get());
    buf_rooms.erase(pos);

    const auto number_of_dynamic_entities = static_cast<size_t>(static_cast<float>(buf_rooms.size()) * dynamic_objets_ratio);
    if (number_of_dynamic_entities > 1) throw std::invalid_argument("GAME_ENEMY_RATIO should be <= 1");
    for (size_t i = 0; i < number_of_dynamic_entities; ++i) {
        m_context.dynamic_objects.emplace_back(std::move(std::make_unique<Enemy>()));
        distrib.param(std::uniform_int_distribution<>::param_type(0, static_cast<int>(buf_rooms.size()) - 1));
        pos = buf_rooms.begin() + distrib(gen);
        m_context.dynamic_objects.back()->set_location((*pos)->get());
        buf_rooms.erase(pos);
    }

    for (const auto& it_room : buf_rooms) {
        m_context.static_objects.emplace_back(std::move(std::make_unique<Food>()));
        m_context.static_objects.back()->set_location(it_room->get());
    }
}

void CommonBuilder::create_state(IStateManager& state_manager, const sf::VideoMode& mode, const std::string& window_title) {
    m_game_state = std::make_unique<GameState>(state_manager, mode, window_title);
}

void CommonBuilder::set_all_to_state() {
    std::vector<std::unique_ptr<Room>> rooms;
    for (auto& row : m_rooms) {
        for (auto& room : row) {
            rooms.emplace_back(std::move(room));
        }
    }
    m_game_state->set_maze(Maze(std::move(rooms)));
    m_game_state->set_context(std::move(m_context));
}
